#include "Soundgin.h"
#include <SoftwareSerial.h>

//Soundgin Data Pins.
//Control Outputs
const int Pin_Sounding = 10; // Sounding
const int Pin_Q = 3;         // Q

//Control Input
const int Pin_Demo = 11;  // Demo. (Active LOW) HIGH = normal operation.
const int Pin_Baud = 12;  // Baud Rate.  9600bps=HIGH / 2400bps=LOW.
const int Pin_Reset = 13; // Reset. (Active LOW) HIGH = normal operation.

// Comm Lines
const int Pin_CTS = 2;    // Clear To Send
const int Pin_RTS = A0;   // Request To Send
const int Pin_TX = A1;    // Transmit
const int Pin_RX = A2;    // Recieve

// No Connect
const int Pin_NC1 = 9;    // No Connect
const int Pin_NC2 = 7;    // No Connect
const int Pin_NC3 = 5;    // No Connect
const int Pin_NC4 = 4;    // No Connect

SoftwareSerial SerialPort(Pin_TX, Pin_RX); // RX, TX  

//Soundgin Phonemes.
enum eSoundPreset{
    Sound_r0 = 128,
    Sound_r1 = 129,
    Sound_r2 = 130,
    Sound_r3 = 131,
    Sound_r4 = 132,
    Sound_r5 = 133,
    Sound_r6 = 134,
    Sound_r7 = 135,
    Sound_r8 = 136,
    Sound_r9 = 137,
    Sound_a0 = 138,
    Sound_a1 = 139,
    Sound_a2 = 140,
    Sound_a3 = 141,
    Sound_a4 = 142,
    Sound_a5 = 143,
    Sound_a6 = 144,
    Sound_a7 = 145,
    Sound_a8 = 146,
    Sound_a9 = 147,
    Sound_b0 = 148,
    Sound_b1 = 149,
    Sound_b2 = 150,
    Sound_b3 = 151,
    Sound_b4 = 152,
    Sound_b5 = 153,
    Sound_b6 = 154,
    Sound_b7 = 155,
    Sound_b8 = 156,
    Sound_b9 = 157,
    Sound_c0 = 158,
    Sound_c1 = 159,
    Sound_c2 = 160,
    Sound_c3 = 161,
    Sound_c4 = 162,
    Sound_c5 = 163,
    Sound_c6 = 164,
    Sound_c7 = 165,
    Sound_c8 = 166,
    Sound_c9 = 167,
    Sound_d0 = 168,
    Sound_d1 = 169,
    Sound_d2 = 170,
    Sound_d3 = 171,
    Sound_d4 = 172,
    Sound_d5 = 173,
    Sound_d6 = 174,
    Sound_d7 = 175,
    Sound_d8 = 176,
    Sound_d9 = 177,
    Sound_d10= 178,
    Sound_d11= 179,
    Sound_m0 = 180,
    Sound_m1 = 181,
    Sound_m2 = 182,
    PH_A  = 192,
    PH_AA = 193,
    PH_AE = 194,
    PH_AU = 195,
    PH_AIR= 196,
    PH_BE = 197,
    PH_BO = 198,
    PH_EB = 199,
    PH_OB = 200,
    PH_CH = 201,
    PH_DE = 202,
    PH_DO = 203,
    PH_ED = 204,
    PH_OD = 205,
    PH_E  = 206,
    PH_EE = 207,
    PH_ER = 208,
    PH_F  = 209,
    PH_GE = 210,
    PH_GO = 211,
    PH_HE = 212,
    PH_HO = 213,
    PH_I  = 214,
    PH_IE = 215,
    PH_J  = 216,
    PH_KE = 217,
    PH_KO = 218,
    PH_EK = 219,
    PH_OK = 220,
    PH_LE = 221,
    PH_LO = 222,
    PH_M  = 223,
    PH_NE = 224,
    PH_NO = 225,
    PH_NGE= 226,
    PH_NGO= 227,
    PH_OE = 228,
    PH_OI = 229,
    PH_OO = 230,
    PH_OU = 231,
    PH_OR = 232,
    PH_PE = 233,
    PH_PO = 234,
    PH_R  = 235,
    PH_SE = 236,
    PH_SO = 237,
    PH_SH = 238,
    PH_T  = 239,
    PH_TH = 240,
    PH_THH= 241,
    PH_TS = 242,
    PH_U  = 243,
    PH_UE = 244,
    PH_V  = 245,
    PH_W  = 246,
    PH_Y  = 247,
    PH_Z  = 248,
    PH_ZH = 249,
    PH_PA0= 250,
    PH_PA1= 251,
    PH_PA2= 252,
    PH_FD0= 253,
    PH_FD1= 254,
    PH_FD2= 255
};

Soundgin::Soundgin(int pin) {
  ledPin   = pin;
  ledState = LOW;
  pinMode(ledPin, OUTPUT);
}

void Soundgin::turnON() {
  ledState = HIGH;
  digitalWrite(ledPin, ledState);
}

void Soundgin::turnOFF() {
  ledState = LOW;
  digitalWrite(ledPin, ledState);
}

int Soundgin::getState() {
  return ledState;
}

void Soundgin::Phoneme(int x){
  SerialPort.write(x);
}


void Soundgin::Say(int x){
  Phoneme(x);
}

void Soundgin::ShutUp(){
  Say(PH_PA1);
}

void Soundgin::TestIO(int PinToTest){
  digitalWrite(PinToTest,LOW);
  delay(20);
  digitalWrite(PinToTest,HIGH);
  delay(20);
}

void Soundgin::Diag(){
  ShutUp();
  //delay(500);
  SayDiag();
  ShutUp();
  delay(500);
  for(int p=0; p<64; p++){
    Say(p);
    ShutUp();
  }
}

void Soundgin::Setup(){
  // Set the Arduino's I/O lines
  pinMode(Pin_Reset, OUTPUT);       digitalWrite(Pin_Reset,LOW);    // Hold reset low while setting up I/O
  pinMode(Pin_Q, INPUT);          
  pinMode(Pin_Sounding, INPUT); 
  pinMode(Pin_Demo, OUTPUT);          digitalWrite(Pin_Demo,HIGH);       
  pinMode(Pin_Baud, OUTPUT);          digitalWrite(Pin_Baud,HIGH);       
  pinMode(Pin_CTS, INPUT);
  pinMode(Pin_TX, INPUT);          
  pinMode(Pin_RTS, OUTPUT);          digitalWrite(Pin_RTS,LOW);       
  pinMode(Pin_RX, OUTPUT);           digitalWrite(Pin_RX,LOW); 
  
  pinMode(Pin_NC1, INPUT);
  pinMode(Pin_NC2, INPUT);
  pinMode(Pin_NC3, INPUT);
  pinMode(Pin_NC4, INPUT);

  SerialPort.begin(9600);

  digitalWrite(Pin_Reset,HIGH);

  Serial.begin(9600);
}

void Soundgin::SayHello() {
  //Say "HELLO"
  Serial.println("Hello");
  Say(PH_HE);
  Say(PH_E);
  Say(PH_LO);
  Say(PH_OE);
  Say(PH_PA2);
}

void Soundgin::SayDiag() {
  //Say "Running Diagnostics"
  Say(PH_R);
  Say(PH_U);
  Say(PH_NE);
  Say(PH_EE);
  Say(PH_NGE);
  Say(PH_PA2);
}
